###PARAMETERS THAT NEED TO BE SET###

#GROUP CONTAINING BANDS TO USE FOR SEGMENTATION AND FEATURE DESCRIPTION
GROUP=l8_vis_ir_mir

#NAME OF OUTPUT RASTER AND VECTOR MAPS
SEGMENT=seg_l8_vis_ir_mir003

#NAME OF VECTOR MAP WITH TRAINING ZONES
TRAINING_ZONES=training_zones

#PATH TO COLOR RULES FILE FOR FINAL COLORS OF CLASSIFIED SEGMENTS
COLOR_RULES=/home/mlennert/ULB/TP/GEOGF425/2013-2014/Segmentation_ClassifRegion/color_rules.txt

#VALUES FOR SEGMENTATION PARAMETERS
THRESHOLD=0.02
MINSIZE=15

#DEFINING THE REGION BOUNDARIES AND RESOLUTION (CAN ALSO BE DONE USING ONE BAND)
g.region -a n=5624160 s=5598480 e=720810 w=684030 res=30 --q

###END OF PARAMETER SECTION###

#SEGMENTATION
echo ""
echo "launching segmentation"
i.segment group=$GROUP out=$SEGMENT thresh=$THRESHOLD minsize=$MINSIZE --o

#TRANSFORM RASTER OF SEGMENTS TO VECTOR USING PIXEL VALUES OF SEGMENTS AS CATEGORY VALUES FOR THE POLYGONS
r.to.vect -v $SEGMENT out=$SEGMENT type=area --o --q

while true; do
    read -p "Segmentation finished. Check whether segmentation results are ok. Continue or stop to change segmentation parameters and relaunch script: (c/s)?" CONT
    case "$CONT" in
        c|C ) break;;
        s|S ) exit 0;;
        * ) echo "Please enter c or s !";;
    esac
done

echo ""

#IMMEDIATELY REMOVE EMPTY LABEL COLUMN
v.db.dropcolumn $SEGMENT col=label

#ADD A COLUMN WHICH WILL HOLD CLASS TYPE INTEGER CODE
v.db.addcolumn $SEGMENT col="class int"

#ADD A BRIGHTNESS COLUMN TO THE ATTRIBUTE TABLE AND SET THE VALUE TO 0
v.db.addcolumn $SEGMENT col="brightness double precision"
v.db.update $SEGMENT col=brightness value=0

#CALCULATE AGGREGATE STATISTICS ON EACH BAND FOR EACH POLYGON AND CALCULATE TOTAL BRIGHTNESS (AS UNWEIGHTED SUM OF MEANS OF ALL BANDS) - AWK AND SED MAGIC DEPENDS ON ORIGINAL BAND NAMES !
#IMMEDIATELY DROP UNNECESSARY COLUMNS
echo "calculating aggregate statistics per segment for each band"
for bande in $(i.group -l $GROUP | tail -n +3 | head -n -1)
   do 
     bande_name=$(echo $bande | sed "s/[<>]//g")
     name=$(echo $bande | awk -F"@" '{print $1}' | awk -F'_' '{print $2}')
     echo "calculating statistics for $bande_name"
     v.rast.stats vector=$SEGMENT raster=${bande_name} col=$name --q
     v.db.update $SEGMENT col=brightness qcolumn="brightness+${name}_mean" --q
     v.db.dropcolumn $SEGMENT col=${name}_n,${name}_min,${name}_max,${name}_range,${name}_cf_var,${name}_sum
   done

#TEXTURE ANALYSIS (HERE GLCM ENTROPY AND HOMOGENEITY IN A 3X3 WINDOW) ON THE PANCHROMATIC BAND (REGION ADJUSTED TO PAN RESOLUTION)
echo ""
echo "calculating texture variables on panchromatique band"
g.region -a res=15
r.texture input=LC81970252013202LGN00_B8@PERMANENT prefix=B8_text method=entr,idm --o --q

#CALCULATE AGGREGATE STATISTICS ON EACH TEXTURE VARIABLE FOR EACH POLYGON
echo ""
echo "calculating aggregate statistics per segment for each texture variable"
for texture in $(g.mlist type=rast pat="B8_text*" --q)
   do 
     name=$(echo $texture | awk -F"_" '{print $3}')
     echo "calculating statistics for $texture"
     v.rast.stats vector=$SEGMENT raster=$texture col=$name --q
     v.db.dropcolumn $SEGMENT col=${name}_n,${name}_min,${name}_max,${name}_range,${name}_cf_var,${name}_sum
   done

#SWITCH BACK TO RESOLUTION OF MULTISPECTRAL BANDS (JUST SO WE DON'T FORGET)
g.region -a res=30

#ADD COLUMNS TO ATTRIBUTE TABLE FOR SHAPE DESCRIPTIO
v.db.addcolumn $SEGMENT col="area int, perimeter int, compact double precision, fract_dim double precision" --q

#CALCULATE SHAPE DESCRIPTION VARIABLES
echo ""
echo "calculating shape description variables"
echo "area"
v.to.db $SEGMENT op=area col=area --q
echo "perimeter"
v.to.db $SEGMENT op=perimeter col=perimeter --q
echo "compactness"
v.to.db $SEGMENT op=compact col=compact --q
echo "fractal dimension"
v.to.db $SEGMENT op=fd col=fract_dim --q

#USE DIGITIZER TO INDICATE CLASS TYPE VALUES FOR SELECTED TRAINING REGIONS

while true; do
    read -p "Before continuing, you need to choose, within the segment vector map, a series of training regions for which you can define their (positive integer) class type using the digitizer (attribute column : class). Press c to continue with existing training zones from a previous run, n to continue with new training zones you just defined or s to stop: (c/n/s)?" CONT
    case "$CONT" in
        c|C ) break;;
        n|N ) v.extract input=$SEGMENT where="class>0" output=${TRAINING_ZONES} --o;; #EXTRACT TRAINING REGIONS TO A SEPERATE VECTOR MAP
        s|S ) exit 0;;
        * ) echo "Please enter c, n or s !";;
    esac
done


#LAUNCH CLASSIFICATION
#NEED TO INDICATE COLUMN NAMES ONLY IF WE DO NOT WANT TO USE ALL COLUMNS
echo "launching classification"
v.class.mlpy input=$SEGMENT training=${TRAINING_ZONES} --q

#EXAMPLE WITH COLUMN NAMES:
#v.class.mlpy input=$SEGMENT training=${TRAINING_ZONES} columns=brightness,B3_mean,B3_stddev,B4_mean,B4_stddev,B5_mean,B5_stddev,B6_mean,B6_stddev,Entr_mean,Entr_stdde,IDM_mean,IDM_stddev,area,perimeter,compact,fract_dim

#LOAD COLOR TABLE BASED ON CLASS TYPE INTO GRASSRGB COLUMN OF SEGMENT VECTOR MAP
v.colors map=$SEGMENT column=class rules=${COLOR_RULES} rgb_column=GRASSRGB --q

#POSSIBLY CONVERT SEGMENT VECTOR TO RASTER WITH CLASSES AND COLORS FOR USE IN MAPSWIPE
while true; do
    read -p "Do you want to transform the segment vector map back to raster (with the same name) to be able to use the map in map swipe ? (y/n)?" CONT
    case "$CONT" in
        y|Y ) break;;
        n|N ) exit 0;;
        * ) echo "Please enter c or s !";;
    esac
done

echo "Creating raster map"
v.to.rast input=$SEGMENT output=$SEGMENT use=attr attrcolumn=class rgbcolumn=GRASSRGB --o --q
